ECA = {}
ECA.Version = '0.0.2'

-- This side of the plugin is going to be dumb: All it does is report the
-- current target distance to the backend.

dofile('tcpsock.lua')

ECA.timer = Timer()

-- States:
ECA.enabled = false
ECA.connected = false
ECA.period_ms = 100
ECA.distance = 250

ECA.update = function()
    local distance = GetTargetDistance()
    if ECA.enabled and ECA.connected then
        if distance ~= nil then
            local data = '[{"distance": ' .. distance .. '}]'
            ECA.conn.tcp:Send(data)
        end
        ECA.timer:SetTimeout(ECA.period_ms, ECA.update)
    end
end

conn_handler = function(conn, err)
    print('Conn handler...')
    if not conn then
        print('Connect failed:')
        print(err)
    else
        print('Connection established.')
        ECA.connected = true
        conn.tcp:Send('[{"set_distance": 250.0}]')
    end
end

line_handler = function(arg)
    print('line handler')
    print(arg)
end

disconn_handler = function(arg)
    print('Connection terminated!')
    ECA.connected = false
    ECA.enabled = false
end

on_write = function(arg)
    if (state == 'idle') then
        print('on_write')
        print(arg)
    end
end

ECA.Connect = function()
    -- Try to connect to local daemon process
    ECA.conn = TCP.make_client('localhost', 9090, conn_handler, line_handler, disconn_handler)
    -- Default distance upon connect is 250
    ECA.distance = 250
end

ECA.Enable = function()
    -- Make sure we are not already enabled
    if ECA.enabled then
        return
    end
    ECA.enabled = true

    -- Make sure we are connected first
    if not ECA.connected then
        print('ECA not connected to daemon! Run "/ECA connect" first.')
        return
    end

    local data = '[{"enable": 1}]'
    ECA.conn.tcp:Send(data)
    ECA.update()
end

ECA.Disable = function()
    ECA.enabled = false
    local data = '[{"enable": 0}]'
    ECA.conn.tcp:Send(data)
end

-- PinDistance: Set the desired distance to the target's current distance
ECA.PinDistance = function()
    -- Get the target's current distance
    local distance = GetTargetDistance()
    if distance ~= nil then
        ECA.SetDistance(distance)
        ECA.Enable()
    end
end

ECA.SetDistance = function(distance)
    if not ECA.connected then
        print('ECA not connected to daemon! Run "/ECA connect" first.')
        return
    end
    
    print('ECA Set distance to: ' .. distance .. 'm')
    ECA.distance = distance
    local data = '[{"set_distance": ' .. tonumber(distance) .. '}]'
    ECA.conn.tcp:Send(data)
end

ECA.SetP = function(p)
    if not ECA.connected then
        print('ECA not connected to daemon! Run "/ECA connect" first.')
        return
    end
    
    local data = '[{"set_p": ' .. tonumber(p) .. '}]'
    ECA.conn.tcp:Send(data)
end

ECA.SetD = function(d)
    if not ECA.connected then
        print('ECA not connected to daemon! Run "/ECA connect" first.')
        return
    end
    
    local data = '[{"set_d": ' .. tonumber(d) .. '}]'
    ECA.conn.tcp:Send(data)
end

ECA.ProcessCmd = function(_, args)
    if args[1] == nil or args[1] == 'help' then 
        print('Commands: ')
        print('/ECA connect: Connect to the local daemon')
        print('/ECA enable: Begin keeping your target at desired distance')
        print('/ECA disable: Disable Combat Assist')
        print('/ECA pin: Enable combat assist pinning the target at its current distance')
        print('/ECA set_p <number>: Set the proportional gain of the control system')
        print('/ECA set_d <number>: Set the derivative gain of the control system. Hint: If your ship overshots the target distance in an oscillating manner, try increasing this value. The default value is 10')
    elseif args[1] == 'connect' then
        ECA.Connect()
    elseif args[1] == 'enable' then
        ECA.Enable()
    elseif args[1] == 'disable' then
        ECA.Disable()
    elseif args[1] == 'pin' then
        ECA.PinDistance()
    elseif args[1] == 'set_distance' then
        if args[2] == nil then 
            print('set_distance requires additional argument')
            return
        end
        local distance = tonumber(args[2])
        if distance ~= nil then
            ECA.SetDistance(distance)
        end
    elseif args[1] == 'set_p' then
        if args[2] == nil then
            print('set_p requires additional argument')
            return
        end
        local p = tonumber(args[2])
        if p ~= nil then
            ECA.SetP(p)
        end
    elseif args[1] == 'set_d' then
        if args[2] == nil then
            print('set_d requires additional argument')
            return
        end
        local d = tonumber(args[2])
        if d ~= nil then
            ECA.SetD(d)
        end
    elseif args[1] == 'change_distance' then
        if args[2] == nil then
            print('change_distance requires additional argument')
            return
        end
        local d = tonumber(args[2])
        if d ~= nil then
            ECA.SetDistance(ECA.distance + d)
        end
    else
        print('Unhandled ECA command:')
        print(args[1])
    end
end

RegisterUserCommand('ECA', ECA.ProcessCmd)
RegisterUserCommand('ECA_enable', ECA.Enable)
RegisterUserCommand('ECA_disable', ECA.Disable)
RegisterUserCommand('ECA_pin', ECA.PinDistance)
